@@ -3,26 +3,39 @@ import os
 import subprocess
 import time
 import traceback
+import weakref
 from abc import abstractmethod
+from datetime import timedelta
 from pathlib import Path
 from threading import Thread, RLock
-from typing import Optional, List, Dict, Collection, cast, TextIO, Union
+from typing import Optional, List, Dict, Collection, cast, Union, IO
 
-from .escape_functions import bash_escape
 from psij.launchers.script_based_launcher import ScriptBasedLauncher
 
 from psij import JobExecutor, JobExecutorConfig, Launcher, Job, SubmitException, \
     JobStatus, JobState
 from psij.executors.batch.template_function_library import ALL as FUNCTION_LIBRARY
 
-
 UNKNOWN_ERROR = 'PSIJ: Unknown error'
 
 logger = logging.getLogger(__name__)
 
 
 def check_status_exit_code(command: str, exit_code: int, out: str) -> None:
-    """Check if exit_code is nonzero and if so raise a RuntimeError."""
+    """Check if exit_code is nonzero and, if so, raise a RuntimeError.
+
+    This function produces a somewhat user-friendly exception message that combines
+    the command that was run with its output.
+
+    Parameters
+    ----------
+    command
+        The command that was run. This is only used to format the error message.
+    exit_code
+        The exit code returned by running the ``command``.
+    out:
+        The output produced by ``command``.
+    """
     if exit_code != 0:
         raise RuntimeError(f'status command {command!r} exited '
                            f'with {exit_code} and output {out!r}')
@@ -38,6 +51,8 @@ def _attrs_to_mustache(job: Job) -> Dict[str, Union[object, List[Dict[str, objec
     for k, v in job.spec.attributes._custom_attributes.items():
         ks = k.split('.', maxsplit=1)
         if len(ks) == 2:
+            # always use lower case here
+            ks[0] = ks[0].lower()
             if ks[0] not in r:
                 r[ks[0]] = []
             cast(List[Dict[str, object]], r[ks[0]]).append({'key': ks[1], 'value': v})
@@ -53,7 +68,7 @@ def _env_to_mustache(job: Job) -> List[Dict[str, str]]:
 
     r = []
     for k, v in job.spec.environment.items():
-        r.append({'name': k, 'value': bash_escape(v)})
+        r.append({'name': k, 'value': v})
     return r
 
 
@@ -69,8 +84,7 @@ class BatchSchedulerExecutorConfig(JobExecutorConfig):
                  initial_queue_polling_delay: int = 2,
                  queue_polling_error_threshold: int = 2,
                  keep_files: bool = False):
-        """Initializes a base batch scheduler executor configuration.
-
+        """
         Parameters
         ----------
         launcher_log_file
@@ -91,7 +105,7 @@ class BatchSchedulerExecutorConfig(JobExecutorConfig):
             Whether to keep submit files and auxiliary job files (exit code and output files) after
             a job has completed.
         """
-        super().__init__(work_directory, launcher_log_file)
+        super().__init__(launcher_log_file, work_directory)
         self.queue_polling_interval = queue_polling_interval
         self.initial_queue_polling_delay = initial_queue_polling_delay
         self.queue_polling_error_threshold = queue_polling_error_threshold
@@ -99,6 +113,13 @@ class BatchSchedulerExecutorConfig(JobExecutorConfig):
         if 'PSIJ_BATCH_KEEP_FILES' in os.environ:
             self.keep_files = True
 
+    @classmethod
+    def _from_config(cls, config: JobExecutorConfig) -> 'BatchSchedulerExecutorConfig':
+        new = cls()
+        new.work_directory = config.work_directory
+        new.launcher_log_file = config.launcher_log_file
+        return new
+
 
 class InvalidJobStateError(Exception):
     """An exception that signals that a job cannot be cancelled due to it being already done."""
@@ -152,6 +173,10 @@ class BatchSchedulerExecutor(JobExecutor):
         2. store the exit code of the launch command in the *exit code file* named
         `<native_id>.ec`, also inside `<script_dir>`.
 
+    Additionally, where appropriate, the submit script should set the environment variable named
+    ``PSIJ_NODEFILE`` to point to a file containing a list of nodes that are allocated for the job,
+    one per line, with a total number of lines matching the process count of the job.
+
     Once the submit script is generated, the executor renders the submit command using
     :func:`~get_submit_command` and executes it. Its output is then parsed using
     :func:`~job_id_from_submit_output` to retrieve the `native_id` of the job. Subsequently, the
@@ -172,8 +197,7 @@ class BatchSchedulerExecutor(JobExecutor):
 
     def __init__(self, url: Optional[str] = None,
                  config: Optional[BatchSchedulerExecutorConfig] = None):
-        """Initializes a `BatchSchedulerExecutor`.
-
+        """
         Parameters
         ----------
         url
@@ -182,7 +206,7 @@ class BatchSchedulerExecutor(JobExecutor):
             An configuration for this executor instance; if none is specified, a default
             configuration is used.
         """
-        super().__init__(url=url, config=config if config else BatchSchedulerExecutorConfig())
+        super().__init__(url=url, config=self._get_config(config))
         assert config
         self.work_directory = config.work_directory / self.name
         self._queue_poll_thread = self._start_queue_poll_thread()
@@ -190,6 +214,13 @@ class BatchSchedulerExecutor(JobExecutor):
     def _ensure_work_dir(self) -> None:
         self.work_directory.mkdir(parents=True, exist_ok=True)
 
+    def _get_config(self, config: Optional[JobExecutorConfig]) -> BatchSchedulerExecutorConfig:
+        if config is None:
+            return BatchSchedulerExecutorConfig()
+        if isinstance(config, BatchSchedulerExecutorConfig):
+            return config
+        return BatchSchedulerExecutorConfig._from_config(config)
+
     def submit(self, job: Job) -> None:
         """See :func:`~psij.JobExecutor.submit`."""
         logger.info('Job %s: submitting', job.id)
@@ -212,6 +243,11 @@ class BatchSchedulerExecutor(JobExecutor):
             self._set_job_status(job, JobStatus(JobState.QUEUED,
                                                 metadata={'native_id': job.native_id}))
         except subprocess.CalledProcessError as ex:
+            if logger.isEnabledFor(logging.DEBUG):
+                with submit_file_path.open('r') as submit_file:
+                    script = submit_file.read()
+                logger.debug('Job %s: submit script is: %s' % (job.id, script))
+
             raise SubmitException(ex.output) from None
 
         self._queue_poll_thread.register_job(job)
@@ -268,7 +304,7 @@ class BatchSchedulerExecutor(JobExecutor):
 
     @abstractmethod
     def generate_submit_script(self, job: Job, context: Dict[str, object],
-                               submit_file: TextIO) -> None:
+                               submit_file: IO[str]) -> None:
         """Called to generate a submit script for a job.
 
         Concrete implementations of batch scheduler executors must override this method in
@@ -355,8 +391,8 @@ class BatchSchedulerExecutor(JobExecutor):
         are two options:
 
         1. Instruct the cancel command to not fail on invalid state errors and have this
-        method always raise a :class:`psij.SubmitException`, since it is only invoked on
-        "other" errors.
+        method always raise a :class:`~psij.exceptions.SubmitException`, since it is only invoked
+        on "other" errors.
 
         2. Have the cancel command fail on both invalid state errors and other errors and
         interpret the output from the cancel command to distinguish between the two and raise
@@ -374,7 +410,7 @@ class BatchSchedulerExecutor(JobExecutor):
         InvalidJobStateError
             Raised if the job cancellation has failed because the job was in a completed or failed
             state at the time when the cancellation command was invoked.
-        psij.SubmitException
+        SubmitException
             Raised for all other reasons.
         """
         pass
@@ -420,6 +456,38 @@ class BatchSchedulerExecutor(JobExecutor):
         """
         pass
 
+    @abstractmethod
+    def get_list_command(self) -> List[str]:
+        """Constructs a command to retrieve the list of jobs known to the LRM for the current user.
+
+        Concrete implementations of batch scheduler executors must override this method. Upon
+        running the command, the output can be parsed with :func:`~parse_list_output`.
+
+        Returns
+        -------
+        A list of strings representing the executable and arguments to invoke in order to obtain
+        the list of jobs the LRM knows for the current user.
+        """
+        pass
+
+    def parse_list_output(self, out: str) -> List[str]:
+        """Parses the output of the command obtained from :func:`~get_list_command`.
+
+        The default implementation of this method assumes that the output has no header and
+        consists of native IDs, one per line, possibly surrounded by whitespace. Concrete
+        implementations should override this method if a different format is expected.
+
+        Parameters
+        ----------
+        out
+            The output from the "list" command as returned by :func:`~get_list_command`.
+        Returns
+        -------
+        A list of strings representing the native IDs of the jobs known to the LRM for the current
+        user.
+        """
+        return [s.strip() for s in out.splitlines()]
+
     def _create_script_context(self, job: Job) -> Dict[str, object]:
         launcher = self._get_launcher_from_job(job)
         if isinstance(launcher, ScriptBasedLauncher) and logger.isEnabledFor(logging.DEBUG):
@@ -439,8 +507,18 @@ class BatchSchedulerExecutor(JobExecutor):
                 'script_dir': str(self.work_directory)
             }
         }
+        assert job.spec is not None
+        if job.spec.attributes:
+            duration = job.spec.attributes.duration
+            if duration is not None:
+                ctx['formatted_job_duration'] = self._format_duration(duration)
         return ctx
 
+    def _format_duration(self, d: timedelta) -> str:
+        # the default is hh:mm:ss, with hh not limited to 24; this is the least ambiguous
+        # choice
+        return '%s:%s:%s' % (int(d.total_seconds()) // 3600, (d.seconds // 60) % 60, d.seconds % 60)
+
     def _run_command(self, cmd: List[str]) -> str:
         res = subprocess.run(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE, text=True)
         if res.returncode != 0:
@@ -478,7 +556,12 @@ class BatchSchedulerExecutor(JobExecutor):
             assert isinstance(self.config, BatchSchedulerExecutorConfig)
             if not self.config.keep_files:
                 submit_file_path = self.work_directory / (job.id + '.job')
-                submit_file_path.unlink()
+                try:
+                    submit_file_path.unlink()
+                except FileNotFoundError:
+                    # this can reasonably happen for attached jobs when the main
+                    # job cleans up the script instead
+                    pass
         except Exception as ex:
             logger.warning('Job %s: failed clean submit script: %s', job.id, ex)
 
@@ -503,7 +586,11 @@ class BatchSchedulerExecutor(JobExecutor):
                 if status.message is None:
                     # only read output from submit script if another error message is not
                     # already present
-                    status.message = self._read_aux_file(job, '.out')
+                    out = self._read_aux_file(job, '.out')
+                    if out:
+                        launcher = self._get_launcher_from_job(job)
+                        if launcher.is_launcher_failure(out):
+                            status.message = launcher.get_launcher_failure_message(out)
                     logger.debug('Output from launcher: %s', status.message)
                 else:
                     self._delete_aux_file(job, '.out')
@@ -540,7 +627,10 @@ class BatchSchedulerExecutor(JobExecutor):
             assert suffix
             path = self.work_directory / (job.native_id + suffix)
         if force or path.exists():
-            path.unlink()
+            try:
+                path.unlink()
+            except FileNotFoundError:
+                pass  # see above; attached job may race with original job
 
     def list(self) -> List[str]:
         """Returns a list of jobs known to the underlying implementation.
@@ -551,7 +641,10 @@ class BatchSchedulerExecutor(JobExecutor):
         Implementations are encouraged to restrict the results to jobs accessible by the current
         user.
         """
-        raise NotImplementedError()
+        return self.parse_list_output(self._run_command(self.get_list_command()))
+
+    def _current_user(self) -> str:
+        return os.getlogin()
 
 
 class _QueuePollThread(Thread):
@@ -561,7 +654,8 @@ class _QueuePollThread(Thread):
         self.name = name
         self.daemon = True
         self.config = config
-        self.executor = executor
+        self.done = False
+        self.executor = weakref.ref(executor, self._stop)
         # native_id -> job
         self._jobs: Dict[str, List[Job]] = {}
         # counts consecutive errors while invoking qstat or equivalent
@@ -569,26 +663,31 @@ class _QueuePollThread(Thread):
         self._jobs_lock = RLock()
 
     def run(self) -> None:
-        logger.debug('Executor %s: queue poll thread started', self.executor)
+        logger.debug('Executor %s: queue poll thread started', self.executor())
         time.sleep(self.config.initial_queue_polling_delay)
-        while True:
+        while not self.done:
             self._poll()
             time.sleep(self.config.queue_polling_interval)
 
+    def _stop(self, exec: object) -> None:
+        self.done = True
+
     def _poll(self) -> None:
+        exec = self.executor()
+        if exec is None:
+            return
         with self._jobs_lock:
             if len(self._jobs) == 0:
                 return
             jobs_copy = dict(self._jobs)
         logger.info('Polling for %s jobs', len(jobs_copy))
         try:
-            out = self.executor._run_command(self.executor.get_status_command(jobs_copy.keys()))
+            out = exec._run_command(exec.get_status_command(jobs_copy.keys()))
         except subprocess.CalledProcessError as ex:
             out = ex.output
             exit_code = ex.returncode
         except Exception as ex:
-            self._handle_poll_error(True,
-                                    ex,
+            self._handle_poll_error(exec, True, ex,
                                     f'Failed to poll for job status: {traceback.format_exc()}')
             return
         else:
@@ -596,10 +695,9 @@ class _QueuePollThread(Thread):
             self._poll_error_count = 0
         logger.debug('Output from status command: %s', out)
         try:
-            status_map = self.executor.parse_status_output(exit_code, out)
+            status_map = exec.parse_status_output(exit_code, out)
         except Exception as ex:
-            self._handle_poll_error(False,
-                                    ex,
+            self._handle_poll_error(exec, False, ex,
                                     f'Failed to poll for job status: {traceback.format_exc()}')
             return
         try:
@@ -611,13 +709,13 @@ class _QueuePollThread(Thread):
                                        message='Failed to update job status: %s' %
                                                traceback.format_exc())
                 for job in job_list:
-                    self.executor._set_job_status(job, status)
+                    exec._set_job_status(job, status)
                 if status.state.final:
                     with self._jobs_lock:
                         del self._jobs[native_id]
         except Exception as ex:
             msg = traceback.format_exc()
-            self._handle_poll_error(True, ex, 'Error updating job statuses {}'.format(msg))
+            self._handle_poll_error(exec, True, ex, 'Error updating job statuses {}'.format(msg))
 
     def _get_job_status(self, native_id: str, status_map: Dict[str, JobStatus]) -> JobStatus:
         if native_id in status_map:
@@ -625,7 +723,8 @@ class _QueuePollThread(Thread):
         else:
             return JobStatus(JobState.COMPLETED)
 
-    def _handle_poll_error(self, immediate: bool, ex: Exception, msg: str) -> None:
+    def _handle_poll_error(self, exec: BatchSchedulerExecutor, immediate: bool, ex: Exception,
+                           msg: str) -> None:
         logger.warning('Polling error: %s', msg)
         self._poll_error_count += 1
         if immediate or (self._poll_error_count > self.config.queue_polling_error_threshold):
@@ -642,7 +741,7 @@ class _QueuePollThread(Thread):
                 self._jobs.clear()
             for job_list in jobs_copy.values():
                 for job in job_list:
-                    self.executor._set_job_status(job, JobStatus(JobState.FAILED, message=msg))
+                    exec._set_job_status(job, JobStatus(JobState.FAILED, message=msg))
 
     def register_job(self, job: Job) -> None:
         assert job.native_id
